/*
Example for AnyNet_2G Click

    Date          : Nov 2017.
    Author        : MikroE Team

Test configuration PIC32 :
    
    MCU                : P32MX795F512L
    Dev. Board         : EasyPIC Fusion v7
    PIC32 Compiler ver : v4.0.0.0

---

Description :

The application is composed of three sections :

- System Initialization - Initializes all necessary GPIO pins, UART used for
the communcation with GSM module and UART used for infromation logging
- Application Initialization - Initializes driver, power on module and sends few
command for the default module configuration
- Application Task - running in parallel core state machine and api state 
machine.

Additional Functions :

All additional functions such as timer initialization and various handlers 
are defined inside API module provided in form of source code.

Notes :

- UART polling not working with current version of firmware - HFC needed.
- In case of usage of other MCUs Timer initialization must be adjusted according to your MCU.

*/

#include "Click_AnyNet_2G_types.h"
#include "Click_AnyNet_2G_config.h"
#include "__anynet2g_api.h"

static uint8_t getCS()
{
    return RC2_bit;
}

static void setRST( uint8_t state )
{
    LATC1_bit = state;
}

static void setPWM( uint8_t state )
{
    LATD0_bit = state;
}

static void setINT( uint8_t state )
{
    LATE8_bit = 0;
}

const T_hal_gpioObj _MIKROBUS_GPIO = 
{
    { 0, setRST, 0, 0, 0, 0, setPWM, setINT, 0, 0, 0, 0 },
    { 0, 0, getCS, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
};

const T_hal_uartObj _MIKROBUS_UART = 
{
    UART2_Write,
    UART2_Read,
    UART2_Data_Ready
};

void systemInit()
{
    TRISC2_bit = 1;
    TRISC1_bit = 0;
    TRISD0_bit = 0;
    TRISE8_bit = 0;

    UART2_Init( 9600 );
    UART5_Init( 115200 );

    Delay_ms( 500 );
}

void applicationInit()
{
// API INIT (TIMER & UART ISR)
    api_anynet2g_init();

// DRIVER INIT
    anynet2g_uartDriverInit((T_ANYNET2G_P)&_MIKROBUS_GPIO, (T_ANYNET2G_P)&_MIKROBUS_UART);
    anynet2g_coreInit( api_anynet2g_default_handler, 1500 );
    
// HANDLERS ASSIGNATION
    anynet2g_setHandler( "+AWSVER", 1500, api_anynet2g_firmwareID_handler );
    anynet2g_setHandler( "+QCCID", 1500, api_anynet2g_queryICCID_handler );
    anynet2g_setHandler( "+AWSPUBOPEN", 1500, api_anynet2g_openTopicAWS_handler );
    anynet2g_setHandler( "+AWSSUBOPEN", 1500, api_anynet2g_openSubscribeAWS_handler );
    anynet2g_setHandler( "+AWSPUBCLOSE", 1500, api_anynet2g_closeTopicAWS_handler );
    anynet2g_setHandler( "+AWSSTATE", 1500, api_anynet2g_checkStateAWS_handler );
    anynet2g_setHandler( "+AWS", 1500, api_anynet2g_newAWS_handler );

// MODULE POWER ON
    anynet2g_hfcEnable( true );
    anynet2g_modulePower( true );

// MODULE INIT
    anynet2g_cmdSingle( "AT" );
    api_anynet2g_firmwareID();
    api_anynet2g_queryICCID();
    
    api_anynet2g_checkStateAWS( _ANYNET2G_GET );
    
// OPEN TOPIC 0 AND PUBLISH
    api_anynet2g_openTopicAWS( _ANYNET2G_SET, "0", "MY_TOPIC" );
    api_anynet2g_publishTopicAWS( _ANYNET2G_SET, "0", "MIKROE", "" );

// CLOSE TOPIC
    //api_anynet2g_closeTopicAWS( _ANYNET2G_SET, "0" );

// SUBSCRIBE 
    //api_anynet2g_openSubscribeAWS( _ANYNET2G_SET, "0", "MYTOP1" );
}

void applicationTask()
{
// CORE STATE MACHINE
    anynet2g_process();
}

void main()
{
    systemInit();
    applicationInit();

    while (1)
    {
        applicationTask();
    }
}